@@ -45,7 +45,6 @@ impl Error {
             ErrorKind::InvalidAttrSelectorModifier => "Invalid attribute modifier".into(),
             ErrorKind::ExpectedNumber => "Expected a number".into(),
             ErrorKind::InvalidSupportQuery => "Invalid support query".into(),
-            ErrorKind::InvalidKeyframeSelector => "Invalid keyframe selector".into(),
             ErrorKind::InvalidMediaQuery => "Invalid media query".into(),
             ErrorKind::InvalidLayerBlockAtRule => "Invalid @layer block at-rule".into(),
             ErrorKind::UnknownAtRuleNotTerminated => "Unknown @rule is not terminated".into(),
@@ -88,7 +87,6 @@ pub enum ErrorKind {
     InvalidAttrSelectorModifier,
     ExpectedNumber,
     InvalidSupportQuery,
-    InvalidKeyframeSelector,
     InvalidLayerBlockAtRule,
     InvalidMediaQuery,
     InvalidAnPlusBMicrosyntax,
