@@ -1,4 +1,4 @@
-use super::{input::ParserInput, traits::ParseDelmited, PResult, Parser};
+use super::{input::ParserInput, PResult, Parser};
 use crate::{
     error::{Error, ErrorKind},
     parser::{Ctx, RuleContext},
@@ -388,12 +388,53 @@ where
 {
     fn parse(&mut self) -> PResult<KeyframesRule> {
         let span = self.input.cur_span()?;
-        let name = match cur!(self) {
+        let name = self.parse()?;
+
+        self.input.skip_ws()?;
+
+        expect!(self, "{");
+
+        self.input.skip_ws()?;
+
+        let mut blocks = vec![];
+
+        loop {
+            if is!(self, "}") {
+                break;
+            }
+
+            self.input.skip_ws()?;
+
+            let block = self.parse()?;
+
+            blocks.push(block);
+
+            self.input.skip_ws()?;
+        }
+
+        expect!(self, "}");
+
+        Ok(KeyframesRule {
+            span: span!(self, span.lo),
+            name,
+            blocks,
+        })
+    }
+}
+
+impl<I> Parse<KeyframesName> for Parser<I>
+where
+    I: ParserInput,
+{
+    fn parse(&mut self) -> PResult<KeyframesName> {
+        match cur!(self) {
             tok!("ident") => {
                 let custom_ident: CustomIdent = self.parse()?;
 
                 match &*custom_ident.value.to_ascii_lowercase() {
                     "none" => {
+                        let span = self.input.cur_span()?;
+
                         return Err(Error::new(
                             span,
                             ErrorKind::InvalidCustomIdent(stringify!(value)),
@@ -402,32 +443,83 @@ where
                     _ => {}
                 }
 
-                KeyframesName::CustomIdent(custom_ident)
+                Ok(KeyframesName::CustomIdent(custom_ident))
             }
-            tok!("str") => KeyframesName::Str(self.parse()?),
-            _ => return Err(Error::new(span, ErrorKind::Expected("ident or string"))),
-        };
-        let mut blocks = vec![];
+            tok!("str") => Ok(KeyframesName::Str(self.parse()?)),
+            _ => {
+                let span = self.input.cur_span()?;
 
-        self.input.skip_ws()?;
+                return Err(Error::new(span, ErrorKind::Expected("ident or string")));
+            }
+        }
+    }
+}
 
-        if is!(self, "{") {
-            expect!(self, "{");
+impl<I> Parse<KeyframeBlock> for Parser<I>
+where
+    I: ParserInput,
+{
+    fn parse(&mut self) -> PResult<KeyframeBlock> {
+        let span = self.input.cur_span()?;
 
-            // TODO: change on `parse_simple_block`
-            blocks = self.parse_delimited(true)?;
+        let child = self.parse()?;
+        let mut prelude = vec![child];
 
-            expect!(self, "}");
+        loop {
+            self.input.skip_ws()?;
+
+            if !eat!(self, ",") {
+                break;
+            }
+
+            self.input.skip_ws()?;
+
+            let child = self.parse()?;
+
+            prelude.push(child);
         }
 
-        Ok(KeyframesRule {
+        let block = self.parse()?;
+
+        Ok(KeyframeBlock {
             span: span!(self, span.lo),
-            name,
-            blocks,
+            prelude,
+            block,
         })
     }
 }
 
+impl<I> Parse<KeyframeSelector> for Parser<I>
+where
+    I: ParserInput,
+{
+    fn parse(&mut self) -> PResult<KeyframeSelector> {
+        match cur!(self) {
+            tok!("ident") => {
+                let ident: Ident = self.parse()?;
+                let lowercased_ident_value = ident.value.to_ascii_lowercase();
+
+                if &*lowercased_ident_value != "from" && &*lowercased_ident_value != "to" {
+                    let span = self.input.cur_span()?;
+
+                    return Err(Error::new(
+                        span,
+                        ErrorKind::Expected("'from' or 'to' idents"),
+                    ));
+                }
+
+                Ok(KeyframeSelector::Ident(ident))
+            }
+            tok!("percent") => Ok(KeyframeSelector::Percent(self.parse()?)),
+            _ => {
+                let span = self.input.cur_span()?;
+
+                return Err(Error::new(span, ErrorKind::Expected("ident or percentage")));
+            }
+        }
+    }
+}
+
 impl<I> Parse<ViewportRule> for Parser<I>
 where
     I: ParserInput,
@@ -722,89 +814,6 @@ where
     }
 }
 
-impl<I> Parse<KeyframeSelector> for Parser<I>
-where
-    I: ParserInput,
-{
-    fn parse(&mut self) -> PResult<KeyframeSelector> {
-        let span = self.input.cur_span()?;
-
-        if is!(self, Ident) {
-            self.parse().map(KeyframeSelector::Ident)
-        } else if is!(self, Percent) {
-            self.parse().map(KeyframeSelector::Percent)
-        } else {
-            Err(Error::new(span, ErrorKind::InvalidKeyframeSelector))
-        }
-    }
-}
-
-impl<I> Parse<KeyframeBlockRule> for Parser<I>
-where
-    I: ParserInput,
-{
-    fn parse(&mut self) -> PResult<KeyframeBlockRule> {
-        if is!(self, AtKeyword) {
-            return self
-                .parse_at_rule(Default::default())
-                .map(Box::new)
-                .map(KeyframeBlockRule::AtRule);
-        }
-
-        self.parse().map(Box::new).map(KeyframeBlockRule::Block)
-    }
-}
-
-impl<I> Parse<KeyframeBlock> for Parser<I>
-where
-    I: ParserInput,
-{
-    fn parse(&mut self) -> PResult<KeyframeBlock> {
-        let span = self.input.cur_span()?;
-
-        let selector = self.parse_delimited(false)?;
-
-        let rule = self.parse()?;
-
-        Ok(KeyframeBlock {
-            span: span!(self, span.lo),
-            selector,
-            rule,
-        })
-    }
-}
-
-impl<I> ParseDelmited<KeyframeSelector> for Parser<I>
-where
-    I: ParserInput,
-{
-    fn eat_delimiter(&mut self) -> PResult<bool> {
-        self.input.skip_ws()?;
-
-        if eat!(self, ",") {
-            self.input.skip_ws()?;
-            Ok(true)
-        } else {
-            Ok(false)
-        }
-    }
-}
-
-impl<I> ParseDelmited<KeyframeBlock> for Parser<I>
-where
-    I: ParserInput,
-{
-    fn eat_delimiter(&mut self) -> PResult<bool> {
-        self.input.skip_ws()?;
-
-        if is!(self, "}") {
-            Ok(false)
-        } else {
-            Ok(true)
-        }
-    }
-}
-
 impl<I> Parse<MediaRule> for Parser<I>
 where
     I: ParserInput,
